/*---------------------------------------------------------------------------*/
/* Action parameters functions and macros for intelligence engine*/
/* author : Olivier Couvreur*/
/* date   : 7/11/1996*/
/* modify : Fabien MORALES*/
/* date   : 29/01/1997  Vertex param*/
/* modify : Olivier COUVREUR*/
/* date   : 10/02/1997  error handling + */
/* modify : Fabien MORALES*/
/* date   : 05/02/1997  Several Vertex params*/
/* modify : Fabien MORALES*/
/* date   : 12/02/1997  Several long params*/
/*---------------------------------------------------------------------------*/
 
#if !defined(__ACTPARAM_H__)
#define __ACTPARAM_H__
 
#include "Convert.h"
#include "Strintel.h"
 
typedef struct tdstActionParam_
{
  tduGetSetParam a_uParam[C_MAX_NB_PARAM];
} tdstActionParam;
 
/* real*/
void fn_vSetActionParamxValue(unsigned char ucN,HIE_tdxHandleToSuperObject p_SuperObjPerso, MTH_tdxReal xValue);
MTH_tdxReal fn_xGetActionParamxValue(unsigned char ucN,HIE_tdxHandleToSuperObject p_SuperObjPerso);
 
/* lips synchro  CGHT*/
void fn_vSetActionParamhLipsSynchro(unsigned char ucN, HIE_tdxHandleToSuperObject p_SuperObjPerso,tdxHandleOfLipsSynchroTable hLipsSynchro);
tdxHandleOfLipsSynchroTable fn_p_stGetActionParamhLipsSynchro(unsigned char ucN, HIE_tdxHandleToSuperObject p_SuperObjPerso);
 
/* constant*/
void fn_vSetActionParamlValue(unsigned char ucN, HIE_tdxHandleToSuperObject p_SuperObjPerso, long lValue);
long fn_lGetActionParamlValue(unsigned char ucN, HIE_tdxHandleToSuperObject p_SuperObjPerso);
 
/* Vertex */
void fn_vSetActionParamp_stVertex(unsigned char ucN, HIE_tdxHandleToSuperObject p_SuperObjPerso, MTH3D_tdstVector *p_stVertex);
MTH3D_tdstVector *fn_p_stGetActionParamp_stVertex(unsigned char ucN, HIE_tdxHandleToSuperObject p_SuperObjPerso);
 
/* Action*/
void fn_vSetActionParamStateValue(unsigned char ucN, HIE_tdxHandleToSuperObject p_SuperObjPerso, tdxHandleToState hState);
tdxHandleToState fn_xGetActionParamStateValue(unsigned char ucN, HIE_tdxHandleToSuperObject p_SuperObjPerso);
 
#endif
 
 
